# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=W0131,W0232,E0213,E0201,F0203,W0201,R0921,W0613
# pylint-version = 0.7.0
#
# Copyright 2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.browser._base

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

from svnmailer_test import base
_tests = []


#### Test Bases                                     ####
########################################################
class TestBase(base.TestBase):
    """ Base class for browser tests """
    LOAD_MODULES = (('_base', 'svnmailer.browser._base'),)

    def getInitialMocks(self):
        """ Returns the initially mocked modules """
        return [
            self._mock_svnmailer_util(),
            self._mock_svnmailer_subversion(),
            self._mock_posixpath(),
            self._mock_urllib(),
            self._mock_urlparse(),
            self._mock_re(),
        ]


#### Test Cases                                     ####
########################################################
class ParsedQueryTestCase(TestBase):
    """ svnmailer.browser._base.ParsedQuery """
    PREFIX = "ParsedQuery"

    def test00general(self):
        """ ParsedQuery is a public object """
        expected = object
        result = self._base.ParsedQuery

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._base),
            "Is not public")

    def test01initial(self):
        """ ._QUERYRE is compiled initially """
        self.mock.result = [self._base.ParsedQuery._QUERYRE._regex.pattern]
        self.expect(r'[&;]')

    def test02init(self):
        """ .__init__ initializes from string """
        inst = self._base.ParsedQuery('a=b&c=d&a=c')
        self.mock.result.extend([inst._keyorder, inst._query_dict, inst._delim])

        self.expect(
            'Regex.split', r'[&;]', 'a=b&c=d&a=c', 0,
            ['a', 'c', 'a'], {'a': ['b', 'c'], 'c': ['d']}, '&'
        )

    def test03init(self):
        """ .__init__ initializes from empty string """
        inst = self._base.ParsedQuery('')
        self.mock.result.extend([inst._keyorder, inst._query_dict, inst._delim])

        self.expect([], {}, '&')

    def test04init(self):
        """ .__init__ catches border cases """
        inst = self._base.ParsedQuery('a=b&c&a=c&', '.')
        self.mock.result.extend([inst._keyorder, inst._query_dict, inst._delim])

        self.expect(
            'Regex.split', '[&;]', 'a=b&c&a=c&', 0,
            ['a', 'c', 'a', ''], {'a': ['b', 'c'], '': [None], 'c': [None]},
                '.'
        )

    def test05init(self):
        """ .__init__ initializes from ParsedQuery """
        inst1 = self._base.ParsedQuery('a=b&c=d;a=c')
        inst2 = self._base.ParsedQuery(inst1)
        inst1._query_dict['a'].append('1')
        inst1._keyorder.append('a')
        self.mock.result = [inst2._query_dict, inst2._keyorder]

        self.expect({'a': ['b', 'c'], 'c': ['d']}, ['a', 'c', 'a'])

    def test06str(self):
        """ .__str__ returns the correct string """
        inst = self._base.ParsedQuery('a=b&c&a=c&', '.')
        self.mock.result = [str(inst)]

        self.expect('a=b.c.a=c.')

    def test07contains(self):
        """ .__contains__ returns the correct result """
        inst = self._base.ParsedQuery('a=b&c&a=c&', '.')
        self.mock.result = ['a' in inst, 'b' in inst, 'c' in inst]

        self.expect(True, False, True)

    def test08getitem(self):
        """ .__getitem__ returns the correct value """
        inst = self._base.ParsedQuery('a=b&c&a=c&', '.')
        self.mock.result = [inst['a'], inst['c']]

        self.expect(['b', 'c'], [None])

    def test09getitem(self):
        """ .__getitem__ raises a KeyError """
        inst = self._base.ParsedQuery('a=b&c&a=c&', '.')
        self.failUnlessRaises(KeyError, lambda x: x['b'], inst)

    def test10remove(self):
        """ .remove removes all given keys and ignores non-existing ones """
        inst = self._base.ParsedQuery('a=b&c&a=c&&d=x&d=y')
        inst.remove(['c', '', 'b', 'd'])
        self.mock.result = [inst._query_dict, inst._keyorder]

        self.expect({'a': ['b', 'c']}, ['a', 'a'])

    def test11add(self):
        """ .add appends all given key-value-pairs """
        inst = self._base.ParsedQuery('a=b&c&a=c&')
        inst.add((('d', 'x'), ('d', 'y'), ('b', 'c'), ('c', 'v')))
        self.mock.result = [inst._query_dict, inst._keyorder]

        self.expect(
            {'a': ['b', 'c'],
             '' : [None],
             'c': [None, 'v'],
             'b': ['c'],
             'd': ['x', 'y']},
            ['a', 'c', 'a', '', 'd', 'd', 'b', 'c']
        )

    def test12modify(self):
        """ .modify calls .remove and .add properly """
        class MyQuery(self._base.ParsedQuery):
            def add(this, toadd):
                self.mock.result.extend(['add', toadd])
            def remove(this, keys):
                self.mock.result.extend(['remove', keys])
        inst = MyQuery('a=b&c=d')
        self.mock.result = []
        inst.modify()
        inst.modify(set = [('x', 'y')])
        inst.modify(add = [('a', 'f')], remove = ['x'], set = [('g', 'h')])

        self.expect(
            'remove', [], 'add', [],
            'remove', ['x'], 'add', [('x', 'y')],
            'remove', ['x', 'g'], 'add', [('a', 'f'), ('g', 'h')]
        )

_tests.append(ParsedQueryTestCase)


class ParsedUrlTestCase(TestBase):
    """ svnmailer.browser._base.ParsedUrl """
    PREFIX = 'ParsedUrl'

    def setUp(self):
        """ Additional Mocks """
        super(ParsedUrlTestCase, self).setUp()

        self.mock.urlparse.urlparse['blahr'] = tuple('123456')
        class MyUrl(self._base.ParsedUrl):
            def _createParsedQuery(this, query):
                return str(query)
        self._myurl = MyUrl

    def test00general(self):
        """ ParsedUrl is a public object """
        expected = object
        result = self._base.ParsedUrl

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._base),
            "Is not public")

    def test01initial(self):
        """ .PARSENAMES is properly filled initially """
        self.mock.result = [self._base.ParsedUrl.PARSENAMES]
        self.expect(('scheme', 'netloc', 'path', 'param', 'query', 'fragment'))

    def test02init(self):
        """ .__init__ initializes from string """
        inst = self._myurl('blahr')
        self.mock.result.extend([
            inst.scheme, inst.netloc, inst.path, inst.param, inst.query,
            inst.fragment
        ])

        self.expect(
            'urlparse.urlparse', 'blahr', '', 1,
            '1', '2', '3', '4', '5', '6'
        )

    def test03init(self):
        """ .__init__ initializes from ParsedUrl """
        inst1 = self._myurl('blahr')
        inst1.scheme = 's'
        inst1.fragment = 'f'
        inst2 = self._myurl(inst1) 
        self.mock.result.extend([
            inst2.scheme, inst2.netloc, inst2.path, inst2.param, inst2.query,
            inst2.fragment
        ])

        self.expect(
            'urlparse.urlparse', 'blahr', '', 1,
            's', '2', '3', '4', '5', 'f'
        )

    def test04str(self):
        """ .__str__ returns unparsed url """
        inst = self._myurl('blahr')
        self.mock.result.append(str(inst))
        
        self.expect(
            'urlparse.urlparse', 'blahr', '', 1,
            'urlparse.urlunparse', '1', '2', '3', '4', '5', '6',
            '1|2|3|4|5|6'
        )

    def test05copy(self):
        """ .copy returns a different instance """
        inst1 = self._myurl('blahr')
        inst2 = inst1.copy()
        self.mock.result.extend([
            inst2.scheme, inst2.netloc, inst2.path, inst2.param, inst2.query,
            inst2.fragment
        ])

        self.failIf(inst1 is inst2, "Inst1 equals Inst2")
        self.expect(
            'urlparse.urlparse', 'blahr', '', 1,
            '1', '2', '3', '4', '5', '6'
        )

    def test06createParsedQuery(self):
        """ ._createParsedQuery returns a ParsedQuery """
        class MyUrl(self._base.ParsedUrl):
            def __init__(this):
                pass
        self.mock.result = [
            MyUrl()._createParsedQuery('some=query').__class__.__name__
        ]

        self.expect('ParsedQuery')

_tests.append(ParsedUrlTestCase)


class TemplateTestCase(TestBase):
    """ svnmailer.browser._base.Template """
    PREFIX = "Template"

    def test00general(self):
        """ Template is a public object """
        expected = object
        result = self._base.Template

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._base),
            "Is not public")

    def test01initial(self):
        """ ._TYPES is properly filled initially """
        self.mock.result = [self._base.Template._TYPES]

        self.expect((
            'revision',
            'deleted',  'deleted_dir',
            'copied',   'copied_dir',
            'added',    'added_dir',
            'modified', 'modified_dir',
        ))

    def test02init(self):
        """ .__init__ initializes templates to None """
        inst = self._base.Template()
        self.mock.result = [inst._templates]

        self.expect({
            'revision': None,
            'deleted' : None, 'deleted_dir' : None,
            'copied'  : None, 'copied_dir'  : None,
            'added'   : None, 'added_dir'   : None,
            'modified': None, 'modified_dir': None,
        })

    def test03init(self):
        """ .__init__ initializes templates from keywords """
        inst = self._base.Template(revision = 'foo', copied_dir = 'bar')
        self.mock.result = [inst._templates]

        self.expect({
            'revision': 'foo',
            'deleted' : None, 'deleted_dir' : None,
            'copied'  : None, 'copied_dir'  : 'bar',
            'added'   : None, 'added_dir'   : None,
            'modified': None, 'modified_dir': None,
        })

    def test04init(self):
        """ .__init__ raises AttributeError for unknown keywords """
        self.failUnlessRaises(AttributeError, self._base.Template, foo = 'bar')

    def test05fromTemplates(self):
        """ .fromTemplates evaluates the given templates properly """
        class MyTemplate(self._base.Template):
            def __init__(this, **kwargs):
                self.mock.result.extend(['__init__', kwargs])
        url = self._base.ParsedUrl('http://opensource.perlig.de/')
        self.mock.result = []
        inst = MyTemplate.fromTemplates(url, {
            'deleted': ('path', {'set': [('a', 'b')]}),
            'added': (None, None),
        })
        self.mock.result.append(inst.__class__.__name__)

        self.expect(
            'posixpath.join', '/', 'path',
            'urlparse.urlunparse', 'http', 'opensource.perlig.de', '//path',
                '', 'a=b', '',
            'urlparse.urlunparse', 'http', 'opensource.perlig.de', '/',
                '', '', '',
            '__init__', {'deleted': 'http|opensource.perlig.de|//path||a=b|',
                'added': 'http|opensource.perlig.de|/|||'},
            'MyTemplate'
        )

    def test06fromTemplates(self):
        """ .fromTemplates passes AttributeError """
        class MyTemplate(self._base.Template):
            def __init__(this, **kwargs):
                raise AttributeError()
        url = self._base.ParsedUrl('http://opensource.perlig.de/')

        self.failUnlessRaises(AttributeError,
            MyTemplate.fromTemplates, url, {
            'deleted': ('path', {'set': [('a', 'b')]}),
            'added': (None, None),
        })

    def test07hasTemplates(self):
        """ .hasTemplates returns True if there are templates """
        class MyTemplate(self._base.Template):
            def __init__(this):
                this._templates = {'foo': None, 'bar': '', 'baz': 'zonk'}
        self.mock.result.append(MyTemplate().hasTemplates())

        self.expect(True)

    def test08hasTemplates(self):
        """ .hasTemplates returns False if there are no templates """
        class MyTemplate(self._base.Template):
            def __init__(this):
                this._templates = {'foo': None, 'bar': '', 'baz': 0}
        self.mock.result.append(MyTemplate().hasTemplates())

        self.expect(False)

    def test09getRevisionUrl(self):
        """ .getRevisionUrl returns the revision url template """
        inst = self._base.Template()
        self.mock.result.append(inst.getRevisionUrl())
        inst = self._base.Template(revision = 'foo')
        self.mock.result.append(inst.getRevisionUrl())

        self.expect(None, 'foo')

    def test10selectByChange(self):
        """ .selectByChange selects 'deleted' """
        change = self.mock['svnmailer.subversion'
            ].VersionedPathDescriptor(1, 'cd')
        inst = self._base.Template(deleted = 'foo')
        self.mock.result.append(inst.selectByChange(change))

        self.expect('foo')

    def test11selectByChange(self):
        """ .selectByChange selects 'deleted_dir' """
        change = self.mock['svnmailer.subversion'
            ].VersionedPathDescriptor(1, 'cdd')
        inst = self._base.Template(deleted_dir = 'bar')
        self.mock.result.append(inst.selectByChange(change))

        self.expect('bar')

    def test12selectByChange(self):
        """ .selectByChange selects 'copied' """
        change = self.mock['svnmailer.subversion'
            ].VersionedPathDescriptor(1, 'cc')
        inst = self._base.Template(copied = 'baz')
        self.mock.result.append(inst.selectByChange(change))

        self.expect('baz')

    def test13selectByChange(self):
        """ .selectByChange selects 'copied_dir' """
        change = self.mock['svnmailer.subversion'
            ].VersionedPathDescriptor(1, 'ccd')
        inst = self._base.Template(copied_dir = 'zonk')
        self.mock.result.append(inst.selectByChange(change))

        self.expect('zonk')

    def test14selectByChange(self):
        """ .selectByChange selects 'added' """
        change = self.mock['svnmailer.subversion'
            ].VersionedPathDescriptor(1, 'ca')
        inst = self._base.Template(added = 'foo')
        self.mock.result.append(inst.selectByChange(change))

        self.expect('foo')

    def test15selectByChange(self):
        """ .selectByChange selects 'added_dir' """
        change = self.mock['svnmailer.subversion'
            ].VersionedPathDescriptor(1, 'cad')
        inst = self._base.Template(added_dir = 'bar')
        self.mock.result.append(inst.selectByChange(change))

        self.expect('bar')

    def test16selectByChange(self):
        """ .selectByChange selects 'modified' """
        change = self.mock['svnmailer.subversion'
            ].VersionedPathDescriptor(1, 'cm')
        inst = self._base.Template(modified = 'baz')
        self.mock.result.append(inst.selectByChange(change))

        self.expect('baz')

    def test17selectByChange(self):
        """ .selectByChange selects 'modified' """
        change = self.mock['svnmailer.subversion'
            ].VersionedPathDescriptor(1, 'cmd')
        inst = self._base.Template(modified_dir = 'zonk')
        self.mock.result.append(inst.selectByChange(change))

        self.expect('zonk')

_tests.append(TemplateTestCase)


class BaseGeneratorTestCase(TestBase):
    """ svnmailer.browser._base.BaseGenerator """
    PREFIX = "BaseGenerator"

    def setUp(self):
        """ sets up test environemt """
        super(BaseGeneratorTestCase, self).setUp()
        class Store(base.Store):
            def __call__(this, arg):
                if arg == 'subst':
                    return {'foo': 'bar', 'baz': 'zonk', u'revision': '2'}
                return None

        class Template(object):
            _revision = True
            _change = True
            def getRevisionUrl(this):
                if this._revision:
                    return "some %(foo)s url template"
                return None
            def selectByChange(this, change):
                if this._change:
                    return "%(baz)s"
                return None

        template = Template()
        class MyGenerator(self._base.BaseGenerator):
            def _createTemplate(this, base_url, config):
                return template
            def _quoteDict(this, toquote):
                self.mock.result.extend(['_quoteDict', toquote])
                return toquote

        self._mygenerator = MyGenerator
        self._template = template
        self._config = Store()

    def test00general(self):
        """ BaseGenerator is a public object """
        expected = object
        result = self._base.BaseGenerator

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._base),
            "Is not public")

    def test01initial(self):
        """ Basegenerator's class attributes is properly set initially """
        cls = self._base.BaseGenerator
        self.mock.result = [
            cls._N_REVISION, cls._N_BASE_REVISION, cls._N_PATH,
            cls._N_BASE_PATH,
            cls._REVISION, cls._BASE_REVISION, cls._PATH, cls._BASE_PATH
        ]

        self.expect(
            u'revision', u'base_revision', u'path', u'base_path',
            u'%(revision)s', u'%(base_revision)s', u'%(path)s',
            u'%(base_path)s'
        )

    def test02init(self):
        """ .__init__ inits the instance attributes and calls _init() """
        class MyGenerator(self._base.BaseGenerator):
            def _createTemplate(this, base_url, config):
                self.mock.result.extend(['_createTemplate', base_url, config])
                return 'the_template'
        inst = MyGenerator('foo', self._config)
        self.mock.result.extend([inst._subst, inst._template])

        self.expect(
            '_createTemplate', 'foo', self._config,
            {'foo': 'bar', 'baz': 'zonk', u"revision": '2'}, 'the_template'
        )

    def test03init(self):
        """ .__init__ passes AttributeError """
        class MyGenerator(self._base.BaseGenerator):
            def _createTemplate(this, base_url, config):
                raise AttributeError()

        self.failUnlessRaises(AttributeError,
            MyGenerator, 'foo', self._config)

    def test04init(self):
        """ .__init__ passes NotImplementedError """
        class MyGenerator(self._base.BaseGenerator):
            def _createTemplate(this, base_url, config):
                raise NotImplementedError()

        self.failUnlessRaises(NotImplementedError,
            MyGenerator, 'foo', self._config)

    def test05init(self):
        """ .__init__ passes browser.Error """
        from svnmailer import browser
        class MyGenerator(self._base.BaseGenerator):
            def _createTemplate(this, base_url, config):
                raise browser.Error()

        self.failUnlessRaises(browser.Error,
            MyGenerator, 'foo', self._config)

    def test06hasTemplates(self):
        """ .hasTemplates returns the proper result """
        class MyGenerator(self._base.BaseGenerator):
            __mocked__ = False
            def __init__(this):
                this._template = base.Store(
                    hasTemplates = lambda: this.__mocked__
                )
        inst = MyGenerator()
        self.mock.result.append(inst.hasTemplates())
        inst.__mocked__ = True
        self.mock.result.append(inst.hasTemplates())

        self.expect(False, True)

    def test07getRevisionUrl(self):
        """ .getRevisionUrl returns the filled URL template """
        inst = self._mygenerator('foooo', self._config)
        self.mock.result.append(inst.getRevisionUrl())

        self.expect(
            '_quoteDict', {'foo': 'bar', 'baz': 'zonk', u"revision": "2"},
            'util.substitute', 'some %(foo)s url template',
                {'foo': 'bar', 'baz': 'zonk', u"revision": "2"},
            'some bar url template'
        )

    def test08getRevisionUrl(self):
        """ .getRevisionUrl returns None """
        self._template._revision = False
        inst = self._mygenerator('foooo', self._config)
        self.mock.result.append(inst.getRevisionUrl())

        self.expect(
            '_quoteDict', {'foo': 'bar', 'baz': 'zonk', u"revision": "2"},
            'util.substitute', None, {'foo': 'bar', 'baz': 'zonk',
                u"revision": "2"},
            None,
        )

    def test09getContentDiffUrl(self):
        """ .getContentDiffUrl returns the filled URL template """
        inst = self._mygenerator('foooo', self._config)
        change = self.mock['svnmailer.subversion'
            ].VersionedPathDescriptor(2, 'c1')
        self.mock.result.append(inst.getContentDiffUrl(change))

        self.expect(
            '_quoteDict', {'baz': 'zonk', u'rev': '2', u'base_rev': u'1',
                u'base_path': u'/somepath', u'base_revision': u'1',
                u'path': u'/otherpath', 'foo': 'bar', u'revision': '2'},
            'util.substitute', '%(baz)s', {'baz': 'zonk', u'rev': '2',
                u'base_rev': u'1', u'base_path': u'/somepath',
                u'base_revision': u'1', u'path': u'/otherpath', 'foo': 'bar',
                u'revision': '2'},
            'zonk'
        )

    def test10getContentDiffUrl(self):
        """ .getContentDiffUrl returns None """
        self._template._change = False
        inst = self._mygenerator('foooo', self._config)
        change = self.mock['svnmailer.subversion'
            ].VersionedPathDescriptor(2, 'c1')
        self.mock.result.append(inst.getContentDiffUrl(change))

        self.expect(None)

    def test11quoteDict(self):
        """ ._quoteDict quotes dict values properly """
        class MyGenerator(self._base.BaseGenerator):
            def __init__(this):
                pass
        toquote = {u'a': u'Andr\xe9', u'b': u'\u20ac'}
        self.mock.result.append(MyGenerator()._quoteDict(toquote))
        self.mock.result.append(MyGenerator()._quoteDict({}))

        self.expect(
            'urllib.quote', 'Andr\xc3\xa9', '/',
            'urllib.quote', '\xe2\x82\xac', '/',
            {u'a': u'quoted(Andr\xe9)', u'b': u'quoted(\u20ac)'},
            {},
        )

    def test12createParsedUrl(self):
        """ ._createParsedUrl returns a ParsedUrl """
        class MyGenerator(self._base.BaseGenerator):
            def __init__(this):
                pass
        class ParsedUrl(object):
            def __new__(cls, toparse):
                return "parsedurl: %s" % toparse
        self._base.ParsedUrl = ParsedUrl
        self.mock.result.append(MyGenerator()._createParsedUrl('foo'))

        self.expect('parsedurl: foo')

    def test13createTemplate(self):
        """ ._createTemplate is not implemented """
        class MyGenerator(self._base.BaseGenerator):
            def __init__(this):
                pass

        self.failUnlessRaises(NotImplementedError,
            MyGenerator()._createTemplate, 'foo', self._config)

_tests.append(BaseGeneratorTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
