# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=E0201,W0131,E0213,F0203,R0912
# pylint-version = 0.7.0
#
# Copyright 2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.settings._fileparser

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

from svnmailer_test import base
_tests = []


#### Test Bases                                     ####
########################################################
class TestBase(base.TestBase):
    """ Base class for settings._fileparser tests """
    LOAD_MODULES = (('_parser', 'svnmailer.settings._fileparser'),)

    def getInitialMocks(self):
        """ Returns the initially mocked modules """
        return [
            self._mock_svnmailer_settings_base(),
        ]


#### Test Cases                                     ####
########################################################
class ErrorTestCase(TestBase):
    """ svnmailer.settings._fileparser.Error """

    def test00general(self):
        """ Error is a public _base.Error """
        expected = self.mock['svnmailer.settings._base'].Error
        result = self._parser.Error

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._parser),
            "Is not public")

_tests.append(ErrorTestCase)


class ParseErrorTestCase(TestBase):
    """ svnmailer.settings._fileparser.ParseError """
    PREFIX = 'ParseError'

    def test00general(self):
        """ ParseError is a public Error """
        expected = self._parser.Error
        result = self._parser.ParseError

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._parser),
            "Is not public")

    def test01initial(self):
        """ ._MESSAGE is properly filled initially """
        cls = self._parser.ParseError
        self.mock.result.append(cls._MESSAGE)

        self.expect("Parse error in %(filename)r, line %(lineno)s")

    def test02init(self):
        """ .__init__ initializes instance attributes """
        inst = self._parser.ParseError('somefile', 42)
        self.mock.result.extend([inst.filename, inst.lineno])

        self.expect('_base.Error', ('somefile', 42), {}, 'somefile', 42)

    def test03str(self):
        """ .__str__ returns the formatted message """
        class MyError(self._parser.ParseError):
            _MESSAGE = 'foo %(filename)r, %(lineno)s'
            def __init__(this):
                this.filename = u"Andr\xe9"
                this.lineno = 23
        inst = MyError()
        self.mock.result.append(str(inst))

        self.expect("foo u'Andr\\xe9', 23")

_tests.append(ParseErrorTestCase)


class ContinuationErrorTestCase(TestBase):
    """ svnmailer.settings._fileparser.ContinuationError """
    PREFIX = 'ContinuationError'

    def test00general(self):
        """ ContinuationError is a public ParseError """
        expected = self._parser.ParseError
        result = self._parser.ContinuationError

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._parser),
            "Is not public")

    def test01initial(self):
        """ ._MESSAGE is properly filled initially """
        cls = self._parser.ContinuationError
        self.mock.result.append(cls._MESSAGE)

        self.expect(
            "Invalid line continuation in %(filename)r, line %(lineno)s"
        )

_tests.append(ContinuationErrorTestCase)


class OptionSyntaxErrorTestCase(TestBase):
    """ svnmailer.settings._fileparser.OptionSyntaxError """
    PREFIX = 'OptionSyntaxError'

    def test00general(self):
        """ OptionSyntaxError is a public ParseError """
        expected = self._parser.ParseError
        result = self._parser.OptionSyntaxError

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._parser),
            "Is not public")

    def test01initial(self):
        """ ._MESSAGE is properly filled initially """
        cls = self._parser.OptionSyntaxError
        self.mock.result.append(cls._MESSAGE)

        self.expect("Option syntax error in %(filename)r, line %(lineno)s")

_tests.append(OptionSyntaxErrorTestCase)


class FileParserTestCase(TestBase):
    """ svnmailer.settings._fileparser.FileParser """
    PREFIX = 'FileParser'

    def setUp(self):
        """ Extends the test environment """
        super(FileParserTestCase, self).setUp()
        class Section(dict):
            def __init__(this, name):
                super(Section, this).__init__()
                this.name = name
            def add(this, option):
                this[option['name']] = option

        class Option(dict):
            def __init__(this, name, value):
                super(Option, this).__init__()
                this.name = name
                this.value = value
                this['name'] = name
                this['value'] = value
            def addLine(this, line):
                this['value'] = '*'.join((this['value'], line))

        class MyParser(self._parser.FileParser):
            def __init__(this):
                pass

        class MyParser2(self._parser.FileParser):
            def __init__(this):
                this._sections = {}
            def _isComment(this, line):
                return bool('comment' in line)
            def _trySectionHeader(this, line):
                if 'section' in line:
                    return line
                return None
            def _parseOption(this, line):
                if 'syntax' in line:
                    return (None, None)
                return line.split()
            def _createSection(this, name = None):
                return Section(name)
            def _createOption(this, name, value):
                return Option(name, value)

        class FakeStream(object):
            def __init__(this, *lines):
                this._lines = ["%s\n" % line for line in lines] + [""]
            def readline(this):
                line = this._lines.pop(0)
                if 'ioerror' in line:
                    raise IOError()
                return line

        self._mystream = FakeStream
        self._parser.Section = Section
        self._parser.Option = Option
        self._myparser = MyParser
        self._myparser2 = MyParser2

    def test00general(self):
        """ FileParser is a public object """
        expected = object
        result = self._parser.FileParser

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._parser),
            "Is not public")

    def test01init(self):
        """ .__init__ initializes ._sections """
        inst = self._parser.FileParser()
        self.mock.result.append(inst._sections)

        self.expect({})

    def test02contains(self):
        """ .__contains__ returns whether the input is an available section """
        inst = self._myparser()
        inst._sections = {'foo': 'sect'}
        self.mock.result.extend(['foo' in inst, 'bar' in inst])

        self.expect(True, False)

    def test03iter(self):
        """ .__iter__ returns a proper iterator (without None section) """
        inst = self._myparser()
        inst._sections = {'foo': 'sect1', 'bar': 'sect2', None: 'sect0'}
        res = {}
        for section in inst:
            res[section] = None
        self.mock.result.append(res)

        self.expect({'sect2': None, 'sect1': None})

    def test04getitem(self):
        """ .__getitem__ returns the specified section """
        inst = self._myparser()
        inst._sections = {'foo': 'sect', 'bar': 'sect2'}
        self.mock.result.extend([inst['foo'], inst['bar']])

        self.expect('sect', 'sect2')

    def test05getitem(self):
        """ .__getitem__ raises KeyError """
        inst = self._myparser()
        inst._sections = {'foo': 'sect'}

        self.failUnlessRaises(KeyError, lambda: inst['bar'])

    def test06delitem(self):
        """ .__delitem__ removes the specified section """
        inst = self._myparser()
        inst._sections = {'foo': 'sect', 'bar': 'sect2'}
        del inst['foo']
        self.mock.result.append(inst._sections)

        self.expect({'bar': 'sect2'})

    def test07delitem(self):
        """ .__delitem__ raises KeyError """
        inst = self._myparser()
        inst._sections = {'bar': 'sect2'}
        def deler(name):
            del inst[name]

        self.failUnlessRaises(KeyError, deler, 'foo')

    def test08slurp(self):
        """ .slurp reads all lines and fills the sections """
        fp = self._mystream(
            "",
            "comment",
            "section1",
            "option1 value1",
            "option2 value2",
            "",
            "section2",
            "option4 value4",
            " value4a",
            "section3",
        )
        inst = self._myparser2()
        inst.slurp(fp, "fakedfile")
        self.mock.result.append(inst._sections)

        self.expect({
            'section1\n': {
                'option2': {'name': 'option2', 'value': 'value2'},
                'option1': {'name': 'option1', 'value': 'value1'}},
            'section2\n': {'option4': {
                'name': 'option4', 'value': 'value4* value4a\n'}},
            'section3\n': {}
        })

    def test09slurp(self):
        """ .slurp fills None section """
        fp = self._mystream(
            "preopt preval",
            "pre2 val2",
            "comment",
            "[section]",
            "opt3 val3",
        )
        inst = self._myparser2()
        inst.slurp(fp, "fake2")
        self.mock.result.append(inst._sections)

        self.expect({
            None: {
                'preopt': {'name': 'preopt', 'value': 'preval'},
                'pre2': {'name': 'pre2', 'value': 'val2'}},
            '[section]\n': {
                'opt3': {'name': 'opt3', 'value': 'val3'}},
        })

    def test10slurp(self):
        """ .slurp raises ContinuationError """
        inst = self._myparser2()

        fp = self._mystream(" foo")
        self.failUnlessRaises(self._parser.ContinuationError,
            inst.slurp, fp, 'fake3')
        fp = self._mystream("[section]", " bar")
        self.failUnlessRaises(self._parser.ContinuationError,
            inst.slurp, fp, 'fake4')
        self.expect(
            '_base.Error', ('fake3', 1), {},
            '_base.Error', ('fake4', 2), {},
        )

    def test11slurp(self):
        """ .slurp raises OptionSyntaxError """
        inst = self._myparser2()
        fp = self._mystream("[section]", "opt value", "syntax")

        self.failUnlessRaises(self._parser.OptionSyntaxError,
            inst.slurp, fp, 'fake5')
        self.expect('_base.Error', ('fake5', 3), {})

    def test12slurp(self):
        """ .slurp passes IOError """
        inst = self._myparser2()
        fp = self._mystream("[section]", "foo bar", "ioerror")

        self.failUnlessRaises(IOError, inst.slurp, fp, 'fake6')

    def test13isComment(self):
        """ ._isComment decides correctly """
        inst = self._myparser()
        self.mock.result.extend([
            inst._isComment('# foo'), inst._isComment('; '),
            inst._isComment(' # fops'), inst._isComment(' ;plop')
        ])

        self.expect(True, True, False, False)

    def test14trySectionHeader(self):
        """ ._trySectionHeader decides properly """
        inst = self._myparser()
        self.mock.result.extend([
            inst._trySectionHeader('[some header]'),
            inst._trySectionHeader('[another header] tralalaa'),
            inst._trySectionHeader('[x]'),
            inst._trySectionHeader('no header'),
            inst._trySectionHeader('also no header]'),
            inst._trySectionHeader('[well...'),
            inst._trySectionHeader('[]'),
        ])

        self.expect(
            'some header', 'another header', 'x', None, None, None, None
        )

    def test15parseOption(self):
        """ ._parseOption decides properly """
        inst = self._myparser()
        self.mock.result.extend([
            inst._parseOption('opt = val'),
            inst._parseOption('opt : val'),
            inst._parseOption('opt=:val'),
            inst._parseOption('opt:=val'),
            inst._parseOption('opt'),
            inst._parseOption(':foo'),
            inst._parseOption('=bar'),
        ])

        self.expect(
            ('opt ', ' val'), ('opt ', ' val'),
            ('opt', ':val'), ('opt', '=val'),
            (None, None), (None, None), (None, None),
        )

    def test16createSection(self):
        """ ._createSection returns a new Section instance """
        inst = self._myparser()
        section = inst._createSection('somename')
        self.failUnless(isinstance(section, self._parser.Section),
            "No Section")
        self.mock.result.extend([section, section.name])

        section = inst._createSection()
        self.failUnless(isinstance(section, self._parser.Section),
            "No Section")
        self.mock.result.extend([section, section.name])

        self.expect({}, 'somename', {}, None)

    def test17createOption(self):
        """ ._createOption returns a new Option instance """
        inst = self._myparser()
        option = inst._createOption('foo', 'bar')
        self.failUnless(isinstance(option, self._parser.Option), "No Option")
        self.mock.result.append(option)

        self.expect({'name': 'foo', 'value': 'bar'})

_tests.append(FileParserTestCase)


class SectionTestCase(TestBase):
    """ svnmailer.settings._fileparser.Section """
    PREFIX = 'Section'

    def setUp(self):
        """ Extends the test environment """
        super(SectionTestCase, self).setUp()
        class MySection(self._parser.Section):
            def __init__(this):
                pass

        self._mysection = MySection

    def test00general(self):
        """ Section is a public object """
        expected = object
        result = self._parser.Section

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._parser),
            "Is not public")

    def test01init(self):
        """ .__init__ initializes instance attributes """
        inst = self._parser.Section('myname')
        self.mock.result.extend([inst.name, inst._options])

        self.expect('myname', {})

    def test02iter(self):
        """ .__iter__ returns the correct iterator """
        inst = self._mysection()
        inst._options = {'foo': 'bar', 'baz': 'zonk'}
        for opt in inst:
            self.mock.result.append(opt)

        self.expect('bar', 'zonk')

    def test03delitem(self):
        """ .__delitem__ deletes the specified option """
        inst = self._mysection()
        inst._options = {'foo': 'bar', 'baz': 'zonk'}
        del inst['foo']
        self.mock.result.append(inst._options)

        self.expect({'baz': 'zonk'})

    def test04delitem(self):
        """ .__delitem__ raises KeyError """
        inst = self._mysection()
        inst._options = {'baz': 'zonk'}
        def deler(name):
            del inst[name]

        self.failUnlessRaises(KeyError, deler, 'foo')

    def test05len(self):
        """ .__len__ returns the number of stored options """
        inst = self._mysection()
        inst._options = {'foo': 'bar', 'baz': 'zonk'}
        self.mock.result.append(len(inst))

        self.expect(2)

    def test06add(self):
        """ .add adds a new option or overrides an existing one """
        inst = self._mysection()
        inst._options = {}
        opt1 = base.Store(name = 'foo', token = 1)
        opt2 = base.Store(name = 'bar', token = 2)
        opt3 = base.Store(name = 'foo', token = 3)
        inst.add(opt1)
        inst.add(opt2)
        inst.add(opt3)
        self.mock.result.append(inst._options)

        self.expect({'foo': opt3, 'bar': opt2})

_tests.append(SectionTestCase)


class OptionTestCase(TestBase):
    """ svnmailer.settings._fileparser.Option """
    PREFIX = 'Option'

    def test00general(self):
        """ Option is a public object """
        expected = object
        result = self._parser.Option

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._parser),
            "Is not public")

    def test01init(self):
        """ .__init__ initializes instance attributes """
        inst = self._parser.Option('optname  ', '  optvalue\t')
        self.mock.result.extend([inst.name, inst.value])

        self.expect('optname', 'optvalue')

    def test02init(self):
        """ .__init__ treats "" special """
        inst = self._parser.Option('optname', ' ""\f')
        self.mock.result.extend([inst.name, inst.value])

        self.expect('optname', '')

    def test03addLine(self):
        """ .addLine adds the supplied line to the value """
        class MyOption(self._parser.Option):
            def __init__(this):
                this.value = 'foo'
        inst = MyOption()
        inst.addLine(' abc ')
        inst.addLine(' def     \n')
        self.mock.result.append(inst.value)

        self.expect('foo abc def')

_tests.append(OptionTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
