# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=E0201,W0131,W0232,E0213,F0203
# pylint-version = 0.7.0
#
# Copyright 2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.settings.mappers

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

from svnmailer_test import base
_tests = []


#### Test Bases                                     ####
########################################################
class TestBase(base.TestBase):
    """ Base class for settings.mappers tests """
    LOAD_MODULES = (('_mappers', 'svnmailer.settings.mappers'),)

    def getInitialMocks(self):
        """ Returns the initially mocked modules """
        return [
            self._mock_svnmailer_settings_base(),
        ]


#### Test Cases                                     ####
########################################################
class ErrorTestCase(TestBase):
    """ svnmailer.settings.mappers.Error """

    def test00general(self):
        """ Error is a public settings.Error """
        expected = self.mock['svnmailer.settings._base'].Error
        result = self._mappers.Error

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._mappers),
            "Is not public")

_tests.append(ErrorTestCase)


class ConfigMappingSectionNotFoundErrorTestCase(TestBase):
    """ svnmailer.settings.mappers.ConfigMappingSectionNotFoundError """

    def test00general(self):
        """ ConfigMappingSectionNotFoundError is a public Error """
        expected = self._mappers.Error
        result = self._mappers.ConfigMappingSectionNotFoundError

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._mappers),
            "Is not public")

_tests.append(ConfigMappingSectionNotFoundErrorTestCase)


class PlainMapperTestCase(TestBase):
    """ svnmailer.settings.mappers.PlainMapper """
    PREFIX = "PlainMapper"

    def setUp(self):
        """ Extends test environment """
        super(PlainMapperTestCase, self).setUp()
        class Config(object):
            def __delitem__(this, section):
                if self.mock.error == 'remove.foo' and section == 'foo':
                    raise KeyError()
                self.mock.result.extend(['Config.__delitem__', section])
            def extractSection(this, section, xform = True, keep = False,
                               check = True):
                if self.mock.error == 'extract.foo':
                    raise self.mock['svnmailer.settings._base'].Error()
                self.mock.result.extend(['Config.extractSection',
                    section, xform, keep, check])
                return [('foo', 'bar'), ('baz', 'zonk')]

        class MyMapper(self._mappers.PlainMapper):
            def __init__(this):
                this._config = Config()
                this._to_remove = []

        self._mymapper = MyMapper

    def test00general(self):
        """ PlainMapper is a public _base.BaseMapper """
        expected = self.mock['svnmailer.settings._base'].BaseMapper
        result = self._mappers.PlainMapper

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._mappers),
            "Is not public")

    def test01init(self):
        """ .init initializes ._to_remove """
        inst = self._mappers.PlainMapper()
        inst.init()
        self.mock.result.append(inst._to_remove)

        self.expect([])

    def test02create(self):
        """ .create returns None for unknown spec """
        self.mock.result.append(self._mymapper().create('foo'))
        self.mock.result.append(self._mymapper().create('[foo'))
        self.mock.result.append(self._mymapper().create('foo]'))

        self.expect(None, None, None)

    def test03create(self):
        """ .create returns a new mapper for proper spec """
        class MyMapper(self._mymapper):
            def _generateMapper(this, section):
                self.mock.result.extend(['_generateMapper', section])
                return "mapper(%s)" % section

        inst = MyMapper()
        self.mock.result.append(inst.create('[bar]'))
        self.mock.result.append(inst._to_remove)

        self.expect('_generateMapper', 'bar', 'mapper(bar)', ['bar'])

    def test04cleanup(self):
        """ .cleanup removes all eaten map sections """
        inst = self._mymapper()
        inst._to_remove.extend(['foo', 'bar'])
        inst.cleanup()
        self.mock.result.append(inst._to_remove)

        self.expect(
            'Config.__delitem__', 'bar',
            'Config.__delitem__', 'foo',
            []
        )

    def test05cleanup(self):
        """ .cleanup catches _base.Errors """
        self.mock.error = 'remove.foo'
        inst = self._mymapper()
        inst._to_remove.extend(['bar', 'foo'])
        inst.cleanup()
        self.mock.result.append(inst._to_remove)

        self.expect('Config.__delitem__', 'bar', [])

    def test06generateMapper(self):
        """ ._generateMapper generates a proper mapper function """
        inst = self._mymapper()
        mapfunc = inst._generateMapper('blub')
        self.mock.result.extend([
            mapfunc('foo'), mapfunc('baz'), mapfunc('plop')
        ])

        self.expect(
            'Config.extractSection', 'blub', False, True, False,
            'bar', 'zonk', 'plop'
        )

    def test07generateMapper(self):
        """ ._generateMapper coverts _base.Error """
        self.mock.error = 'extract.foo'
        inst = self._mymapper()

        self.failUnlessRaises(self._mappers.ConfigMappingSectionNotFoundError,
            inst._generateMapper, 'blub')

_tests.append(PlainMapperTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
