# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=C0103,W0703,W0612,W0302,W0232,W0131,C0102,E0201,C0101
# pylint: disable-msg=W0613,E0213,W0142
# pylint-version = 0.7.0
#
# Copyright 2004-2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.util

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

import unittest
from svnmailer_test import base
_tests = []

def findTestData():
    """ finds the testdata directory """
    import os
    data_path = os.environ.get('TESTDATA')
    if data_path:
        data_path = os.path.abspath(data_path)
        if os.path.isdir(data_path):
            return data_path

    return None


class importTestCase(unittest.TestCase):
    """ Importing svnmailer.util """

    def testImport(self):
        """ from svnmailer import util """
        try:
            from svnmailer import util
        except Exception, exc:
            self.fail("svnmailer.util could not be imported: %s" % str(exc))

_tests.append(importTestCase)


class utilTestBase(base.TestBase):
    """ Base class for util tests """

    def setUp(self):
        """ import the util module """
        super(utilTestBase, self).setUp()

        from svnmailer import util
        reload(util)
        self._util = util


class SafeDictTestCase(utilTestBase):
    """ svnmailer.util.SafeDict """

    def test00KeyError(self):
        """ SafeDict returns empty strings on unknown keys and false values """
        mydict = self._util.SafeDict(
            {'a': 1, 'b': 0, 'c': None, 'd': self, 'e': False}
        )
        tests = (
            ('a', 1), ('b', ''), ('c', ''), ('d', self), ('e', ''), ('z', ''),
        )

        for key, expected in tests:
            result = mydict[key]
            self.failUnlessEqual(expected, result,
                "key %r expected %r, got %r" % (key, expected, result)
            )

_tests.append(SafeDictTestCase)


class ReadOnlyDictTestCase(utilTestBase):
    """ svmailer.util.ReadOnlyDict """

    def test00Init(self):
        """ ReadOnlyDict is initializable """
        mydict = self._util.ReadOnlyDict({'a': 1})

    def test01setitem(self):
        """ ReadOnlyDict raises TypeError when assigning a key """
        mydict = self._util.ReadOnlyDict({'a': 1})
        for key in ['a', 'b']:
            try:
                mydict[key] = 2
            except TypeError:
                """ cool """
                pass
            else:
                self.fail(
                    "Assigning key %r doesn't raise TypeError" % key
                )

    def test02delitem(self):
        """ ReadOnlyDict raises TypeError when deleting a key """
        mydict = self._util.ReadOnlyDict({'a': 1})
        for key in ['a', 'b']:
            try:
                del mydict[key]
            except TypeError:
                """ cool """
                pass
            else:
                self.fail(
                    "Deleting key %r doesn't raise TypeError" % key
                )

    def test03clear(self):
        """ ReadOnlyDict raises TypeError when being cleared """
        mydict = self._util.ReadOnlyDict({'a': 1})
        try:
            mydict.clear()
        except TypeError:
            """ cool """
            pass
        else:
            self.fail("Clearing doesn't raise TypeError")

    def test04fromkeys(self):
        """ ReadOnlyDict.fromkeys doesn't raise an TypeError """
        mydict = self._util.ReadOnlyDict.fromkeys(['a', 'b'], 1)
        try:
            mydict.clear()
        except TypeError:
            """ cool """
            pass
        else:
            self.fail(
                "ReadOnlyDict.fromkeys() doesn't return a read-only dict"
            )
        expected = {'a': 1, 'b': 1}
        result = mydict
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test05pop(self):
        """ ReadOnlyDict raises TypeError when being popped """
        mydict = self._util.ReadOnlyDict({'a': 1})
        for key in ['a', 'b', 'c']:
            try:
                if key == 'c':
                    mydict.pop(key, 3)
                else:
                    mydict.pop(key)
            except TypeError:
                """ cool """
                pass
            else:
                self.fail(
                    "Popping key %r doesn't raise TypeError" % key
                )

    def test06popitem(self):
        """ ReadOnlyDict raises TypeError when being popitem'd """
        mydict = self._util.ReadOnlyDict({'a': 1})
        try:
            mydict.popitem()
        except TypeError:
            """ cool """
            pass
        else:
            self.fail("Popiteming doesn't raise TypeError")

    def test07setdefault(self):
        """ ReadOnlyDict raises TypeError when being setdefault'd """
        mydict = self._util.ReadOnlyDict({'a': 1})
        for key in ['a', 'b']:
            try:
                mydict.setdefault(key, 2)
            except TypeError:
                """ cool """
                pass
            else:
                self.fail(
                    "Setdefaulting key %r doesn't raise TypeError" % key
                )

    def test08update(self):
        """ ReadOnlyDict raises TypeError when being updated """
        mydict = self._util.ReadOnlyDict({'a': 1})
        try:
            mydict.update({'a': 1, 'b': 2})
        except TypeError:
            """ cool """
            pass
        else:
            self.fail("Updating doesn't raise TypeError")

_tests.append(ReadOnlyDictTestCase)


class parseContentTypeTestCase(utilTestBase):
    """ svnmailer.util.parseContentType """

    def test008bit(self):
        """ parseContentType rejects non-ascii input """
        expected = None
        result = self._util.parseContentType('Andr\xe9/Malo')
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test01format(self):
        """ parseContentType rejects malicious strings """
        expected = None
        result = self._util.parseContentType('TEXT')
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02lowertype(self):
        """ parseContentType lowers the type """
        expected = ("text/plain", {})
        result = self._util.parseContentType("Text/Plain")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test03simple(self):
        """ parseContentType accepts simple parameters """
        expected = ('t/p', {'key1': ['val1'], 'key2': ['val2']})
        result = self._util.parseContentType('t/p;key2 =val2; key1=val1 ')
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test04quoted(self):
        """ parseContentType accepts quoted values """
        expected = ('t/p', {'key1': ['val" 1'], 'key2': ['val2 ']})
        result = self._util.parseContentType(
            't/p; key2="val2 "; key1="val\\" 1"'
        )
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(parseContentTypeTestCase)


class inheritTestCase(utilTestBase):
    """ svnmailer.util.inherit """

    def test00inherit(self):
        """ inherit does runtime inheritance """
        expected = ['Derived', 'base', 'derived']
        class CBase(object):
            def foo(self):
                return "Base"
            def bar(self):
                return "base"

        class CDerived(object):
            def foo(self):
                return "Derived"
            def baz(self):
                return "derived"

        cls = self._util.inherit(CDerived, CBase)
        inst = cls()
        result = [inst.foo(), inst.bar(), inst.baz()]
        self.failUnlessEqual(expected, result,
            "expected %r, got %r" % (expected, result)
        )

    def test01private(self):
        """ inherit sets private __decorator_class attribute properly """
        class CBase(object):
            def foo(self):
                return self.__decorator_class
        class CDerived(object):
            def bar(self):
                return self.__decorator_class

        cls = self._util.inherit(CDerived, CBase)
        inst = cls()

        result = inst.bar()
        self.failUnless(
            cls is result,
            "expected %r, got %r" % (cls, result)
        )

        try:
            result = inst.foo()
        except AttributeError:
            """ fine. """
            pass
        else:
            self.fail(
                "Accessing the private attribute in base class doesn't raise "
                "AttributeError"
            )

_tests.append(inheritTestCase)


class commonPathsTestCase(utilTestBase):
    """ svnmailer.util.commonPaths """

    def setUp(self):
        """ set up """
        super(commonPathsTestCase, self).setUp()
        self.addMock('posixpath', {})

    def test00onepath(self):
        """ commonPaths just returns one path """
        expected = ('', ["foo/bar"])
        result = self._util.commonPaths(["foo/bar"])
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test01root(self):
        """ commonPaths just returns if root (/) appears """
        expected = ('', ["foo/bar", "/", "foo/baz"])
        result = self._util.commonPaths(["foo/bar", "/", "foo/baz"])
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test02even(self):
        """ commonPaths deals with cut at slash """
        def prefix(m):
            return "foo/"
        self.mock['posixpath'].commonprefix = prefix

        expected = ("foo", ["./", "bar/", "bar/zonk"])
        result = self._util.commonPaths(["foo/", "foo/bar/", "foo/bar/zonk"])
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test03odd(self):
        """ commonPaths deals with first-item cut at non-slash """
        def prefix(m):
            return "foo"
        self.mock['posixpath'].commonprefix = prefix

        expected = ("", ["foo/", "foo/bar/", "foo/bar/zonk"])
        result = self._util.commonPaths(["foo/", "foo/bar/", "foo/bar/zonk"])
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test04odd(self):
        """ commonPaths deals with next-item cut at non-slash """
        def prefix(m):
            return "foo/bar"
        self.mock['posixpath'].commonprefix = prefix

        expected = ("foo", ["bar_doh", "bar/", "bar/zonk"])
        result = self._util.commonPaths(
            ["foo/bar_doh", "foo/bar/", "foo/bar/zonk"]
        )
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(commonPathsTestCase)


class getGlobValueTestCase(utilTestBase):
    """ svnmailer.util.getGlobValue """

    def test00None(self):
        """ getGlobValue returns None if no match is found """
        tests = (
            ('/foo', []),
            ('/foo', [('*.xml', True)]),
        )
        expected = None
        for test, globs in tests:
            result = self._util.getGlobValue(globs, test)
            self.failUnlessEqual(result, expected,
                "%r: Expected %r, got %r" % (test, expected, result)
            )

    def test01Match(self):
        """ getGlobValue returns the proper value """
        tests = (
            (1, '/foo', [('/foo', 1)]),
            (2, '/foo', [('*foo', 2)]),
            (3, '/foo', [('*bar', 4), ('*foo', 3), ('*foo', 2)]),
        )
        for expected, test, globs in tests:
            result = self._util.getGlobValue(globs, test)
            self.failUnlessEqual(result, expected,
                "%r: Expected %r, got %r" % (test, expected, result)
            )

_tests.append(getGlobValueTestCase)


class getParentDirListTestCase(utilTestBase):
    """ svnmailer.util.getParentDirList """

    def test00Input(self):
        """ getParentDirList traverses up the parent path """
        tests = (
            (['/foo', '/'], '/foo/bar'),
            (['/foo', '/'], '/foo/bar/'),
            (['/foo/bar', '/foo', '/'], '/foo/bar/baz'),
            (['/'], '/'),
            (['/'], '//'),
            (['/wooot', '/'], '//wooot//zonk'),
        )
        for expected, test in tests:
            result = self._util.getParentDirList(test)
            self.failUnlessEqual(result, expected,
                "%r: Expected %r, got %r" % (test, expected, result)
            )

_tests.append(getParentDirListTestCase)


class filterForXmlTestCase(utilTestBase):
    """ svnmailer.util.filterForXml """

    def test00Input(self):
        """ filterForXml replaces invalid characters """
        test = u'\x00a\x01b\x02c\x03d\x04e\x05f\x06g\x07h' \
               u'\x08i\x09j\x0Ak\x0Bl\x0Cm\x0Dn\x0Eo\x0Fp' \
               u'\x10q\x11r\x12s\x13t\x14u\x15v\x16w\x17x' \
               u'\x18y\x19z\x1A0\x1B1\x1C2\x1D3\x1E4\x1F5'
        expected = \
            u'\ufffda\ufffdb\ufffdc\ufffdd\ufffde\ufffdf' \
            u'\ufffdg\ufffdh\ufffdi\u0009j\u000Ak\ufffdl' \
            u'\ufffdm\u000dn\ufffdo\ufffdp\ufffdq\ufffdr' \
            u'\ufffds\ufffdt\ufffdu\ufffdv\ufffdw\ufffdx' \
            u'\ufffdy\ufffdz\ufffd0\ufffd1\ufffd2\ufffd3' \
            u'\ufffd4\ufffd5'
        result = self._util.filterForXml(test)
        self.failUnlessEqual(result, expected,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(filterForXmlTestCase)


class substituteTestCase(utilTestBase):
    """ svnmailer.util.substitute """

    def test00None(self):
        """ substitute accepts and returns None """
        expected = None
        result = self._util.substitute(None, {1: 2})
        self.failUnlessEqual(result, expected,
            "Expected %r, got %r" % (expected, result)
        )
        expected = ""
        result = self._util.substitute("%(a)s", {'a': None})
        self.failUnlessEqual(result, expected,
            "Expected %r, got %r" % (expected, result)
        )

    def test01ValidInput(self):
        """ substitute accepts valid input """
        expected = 'abb'
        result = self._util.substitute('a%(a)sb', {'a': 'b'})
        self.failUnlessEqual(result, expected,
            "Expected %r, got %r" % (expected, result)
        )

    def test02IncompleteInput(self):
        """ substitute accepts incomplete input """
        expected = 'ab'
        result = self._util.substitute('a%(a)s%(b)s', {'a': 'b'})
        self.failUnlessEqual(result, expected,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(substituteTestCase)


class extractX509UserTestCase(utilTestBase):
    """ svnmailer.util.extractX509User """

    def test00ValidInput(self):
        """ extractX509User extracts from X509 strings """
        expected = (u'foo', u'bar')
        result = self._util.extractX509User(
            'zonk/CN=foo/blahr/emailAddress=bar'
        )
        self.failUnlessEqual(result, expected,
            "Expected %r, got %r" % (expected, result)
        )

    def test01InvalidInput(self):
        """ extractX509User gives up on unknown strings """
        expected = None
        result = self._util.extractX509User(
            'zonk/foo/blahr/emailAddress='
        )
        self.failUnlessEqual(result, expected,
            "Expected %r, got %r" % (expected, result)
        )

    def test02IncompleteInput(self):
        """ extractX509User deals with incomplete X509 strings """
        expected = (None, u"foo")
        result = self._util.extractX509User(
            'zonk/blahr/emailAddress=foo'
        )
        self.failUnlessEqual(result, expected,
            "Expected %r, got %r" % (expected, result)
        )
        expected = None
        result = self._util.extractX509User(
            'zonk/CN=bar/blahr'
        )
        self.failUnlessEqual(result, expected,
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(extractX509UserTestCase)


class LocaleFileTestCase(utilTestBase):
    """ svnmailer.util._LocaleFile """

    def test00init(self):
        """ _LocaleFile is initialized properly """
        class _path:
            supports_unicode_filenames = "foo"
        class _os:
            path = _path()
        _os = _os()
        class _locale:
            coding = "iso-2022-jp-invalid"
            def getpreferredencoding(this, dosetlocale):
                return this.coding
        _locale = _locale()
        class _sys:
            coding = "iso-8859-2-invalid"
            def getfilesystemencoding(this):
                return this.coding
        _sys = _sys()

        kwargs = {'_locale': _locale, '_os': _os, '_sys': _sys}
        inst = self._util._LocaleFile(**kwargs)
        expected = ["foo", "iso-2022-jp-invalid", "iso-8859-2-invalid"]
        result   = [inst.unicode_system, inst.from_enc, inst.to_enc] 
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

        _sys.coding = None
        inst = self._util._LocaleFile(**kwargs)
        expected = ["foo", "iso-2022-jp-invalid", "us-ascii"]
        result   = [inst.unicode_system, inst.from_enc, inst.to_enc] 
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

        _locale.coding = None
        inst = self._util._LocaleFile(**kwargs)
        expected = ["foo", "us-ascii", "us-ascii"]
        result   = [inst.unicode_system, inst.from_enc, inst.to_enc] 
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test01toLocale(self):
        """ _LocaleFile.toLocale returns unicode on unicode system """
        inst = self._util._LocaleFile()
        inst.unicode_system = True

        expected = u"\u20acuro"
        result = inst.toLocale(u"\u20acuro")
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        inst.from_enc = "iso-8859-15"
        result = inst.toLocale("\xa4uro")
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        inst.from_enc = "us-ascii"
        result = inst.toLocale("\xa4uro", "iso-8859-15")
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

    def test02toLocale(self):
        """ _LocaleFile.toLocale raises UnicodeError on unicode system """
        inst = self._util._LocaleFile()
        inst.unicode_system = True

        try:
            inst.toLocale("\xa4uro", "us-ascii")
        except UnicodeError:
            """ fine """
            pass
        else:
            self.fail("Doesn't raise UnicodeError")

    def test03toLocale(self):
        """ _LocaleFile.toLocale returns string on string system """
        inst = self._util._LocaleFile()
        inst.unicode_system = False
        inst.from_enc = "iso-8859-15"

        expected = "\xe2\x82\xacuro"

        inst.to_enc = "utf-8"
        result = inst.toLocale("\xa4uro")
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        inst.to_enc = "us-ascii"
        result = inst.toLocale("\xa4uro", locale_enc = "utf-8")
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

    def test04toLocale(self):
        """ _LocaleFile.toLocale raises UnicodeError on string system """
        inst = self._util._LocaleFile()
        inst.unicode_system = False

        try:
            inst.toLocale("\xa4uro", "iso-8859-15", "us-ascii")
        except UnicodeError:
            """ fine """
            pass
        else:
            self.fail("Doesn't raise UnicodeError")

    def test05toLocale(self):
        """ _LocaleFile.toLocale raises AssertionError on string system """
        inst = self._util._LocaleFile()
        inst.unicode_system = False

        try:
            inst.toLocale(u"\xa4uro", locale_enc = "none")
        except AssertionError:
            """ fine """
            pass
        else:
            self.fail("Doesn't raise AssertionError")

    def test06fromLocale(self):
        """ _LocaleFile.fromLocale returns unicode on unicode input """
        inst = self._util._LocaleFile()

        expected = u"\u20acuro"
        result = inst.fromLocale(u"\u20acuro")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test07fromLocale(self):
        """ _LocaleFile.fromLocale returns unicode on string input """
        inst = self._util._LocaleFile()

        expected = u"\u20acuro"
        result = inst.fromLocale("\xa4uro", "iso-8859-15")
        self.failUnlessEqual(expected, result,
            "1st: Expected %r, got %r" % (expected, result)
        )

        inst.from_enc = "iso-8859-15"
        result = inst.fromLocale("\xa4uro")
        self.failUnlessEqual(expected, result,
            "2nd: Expected %r, got %r" % (expected, result)
        )

    def test08fromLocale(self):
        """ _LocaleFile.fromLocale returns string on string input """
        inst = self._util._LocaleFile()

        expected = "\xa4uro"
        result = inst.fromLocale("\xa4uro", "none")
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )

    def test09fromLocale(self):
        """ _LocaleFile.fromLocale raises UnicodeError on string input """
        inst = self._util._LocaleFile()

        try:
            inst.fromLocale("\xa4uro", "us-ascii")
        except UnicodeError:
            """ fine """
            pass
        else:
            self.fail("Doesn't raise UnicodeError")

    def test10filename(self):
        """ filename is a _LocaleFile instance """
        expected = self._util._LocaleFile
        result = self._util.filename
        self.failUnless(isinstance(result, expected),
            "Expected %r, got %r" % (expected, result)
        )

_tests.append(LocaleFileTestCase)


class splitCommandTestCase(utilTestBase):
    """ svnmailer.util.splitCommand """

    def test00InvalidInput(self):
        """ Command split with invalid input raises ValueError """
        cmds = (
            # empty
            None, '', ' ', '\t', '\n', '  \t  \n',
            # otherwise invalid
            '"foo', r'"foo\n"', r'"foo\"',
        )
        for cmd in cmds:
            try:
                cmd_list = self._util.splitCommand(cmd)
            except ValueError:
                """ all cool """
                pass
            else:
                self.fail(
                    "Command string %r doesn't raise ValueError" % cmd
                )

    def test01ValidInput(self):
        """ Command split with valid input works """
        tests = (
            # one simple item
            (
                ['foo'],
                ['foo', ' foo', 'foo ', ' foo\t ']
            ),
            # two simple items
            (
                ['foo', 'bar'],
                ['foo bar', ' foo bar', 'foo bar ', ' foo\t bar \n']
            ),
            # three simple items
            (
                ['foo', 'bar', 'baz'],
                ['foo bar baz', ' foo   bar\t\n  baz  '],
            ),
            # one simple quoted item
            (
                ['foo'],
                ['"foo"', ' "foo"', '"foo" ', ' "foo" ']
            ),
            # item containing WS
            (
                ['f o o'],
                ['"f o o"', ' "f o o"', '"f o o" ', ' "f o o" ']
            ),
            # item containing quotes
            (
                ['foo"'],
                ['foo" ', r'"foo\""']
            ),
            # item containing more escapes and stuff
            (
                [r'"f oo\n" bar', r'ba\\"z"'],
                [r'"\"f oo\\n\" bar" "ba\\\\\"z\""']
            ),
        )
        for expected, cmds in tests:
            for cmd in cmds:
                result = self._util.splitCommand(cmd)
                self.failUnlessEqual(result, expected,
                    "Command string %r yields %r" % (cmd, result)
                )

_tests.append(splitCommandTestCase)


class TempFileTestCase(utilTestBase):
    """ svnmailer.util.TempFile """

    def test00CreationRemoval(self):
        """ Tempfile creation and removal works """
        import os

        tmp = self._util.TempFile()
        name = tmp.name

        self.failUnless(name, "Tempfile name is empty")
        self.failUnless(tmp.fp, "Tempfile descriptor is empty")

        self.failUnless(
            os.path.isfile(name),
            "Tempfile %r was not created" % name
        )

        tmp.close()
        self.failUnless(
            os.path.isfile(name),
            "Tempfile %r was removed after close" % name
        )

        del tmp
        self.failIf(
            os.path.isfile(name),
            "Tempfile %r was not removed in destructor" % name
        )

    if not findTestData():
        def test01TempDirSelection(self):
            """ (no testdata directory) Tempdir selection SKIPPED """
            pass
    else:
        def test01TempDirSelection(self):
            """ Tempdir selection works """
            import os

            otherdir = os.path.join(findTestData(), 'tmp')
            tmp = self._util.TempFile(otherdir)
            self.failUnlessEqual(
                os.path.dirname(tmp.name), otherdir,
                "Supplied tempdir %r was not used, but the tempfile "
                "is %r" % (otherdir, tmp.name)
            )

            tmp = self._util.TempFile()
            self.failIfEqual(
                os.path.dirname(tmp.name), otherdir,
                "No tempdir was supplied, but *something* was used: %r" %
                tmp.name
            )

    def test02OpenModeSelection(self):
        """ Tempfile open mode selection works """
        tmp = self._util.TempFile()
        self.failUnlessEqual(
            tmp.fp.mode, "w+b",
            "Tempfile was not opened in binary mode, but %r" % tmp.fp.mode
        )

        tmp = self._util.TempFile(text = True)
        self.failUnlessEqual(
            tmp.fp.mode, "w+",
            "Tempfile was not opened in text mode, but %r" % tmp.fp.mode
        )

    def test03WriteClose(self):
        """ Tempfile write and close works """
        import random

        charlist = [
            char for char in "abcdefghijklmnopqrstuvwxyz0123456789,;.:-_"
        ]
        random.shuffle(charlist)
        expected = ''.join(charlist)

        tmp = self._util.TempFile()
        tmp.fp.write(expected)
        tmp.close()

        try:
            tmp.fp.write("foo")
        except ValueError:
            """ cool """
            pass
        else:
            self.fail("Write after close didn't raise ValueError")

        content = file(tmp.name, 'rb').read()
        self.failUnlessEqual(
            expected, content,
            "Tempfile doesn't contain the stuff written (%r), but %r" %
            (expected, content)
        )

_tests.append(TempFileTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
